/*
 *	Hippogryf -- Main Program
 *
 *	(c) 2018--2022 Martin Mares <mj@ucw.cz>
 */

#include "hippogryf.h"

#include <getopt.h>

static void usage(void)
{
	cerr << "Usage: hippogryf [<options>>] <src> <test.in>\n";
	cerr << "\n";
	cerr << "Options:\n";
	cerr << "-d	enable debugging\n";
	cerr << "-r	show all registers at the end\n";
	exit(1);
}

int main(int argc, char **argv)
{
	int opt;
	int opt_debug = 0;
	int opt_regs = 0;

	while ((opt = getopt(argc, argv, "dr")) >= 0) {
		switch (opt) {
			case 'd':
				opt_debug = 1;
				break;
			case 'r':
				opt_regs = 1;
				break;
			default:
				usage();
		}
	}

	if (optind + 2 != argc)
		usage();
	const char *src_name = argv[optind];
	const char *in_name = argv[optind+1];

	program pgm;

	try {
		pgm.load(src_name);
		pgm.resolve_labels();
	} catch (hippo_error &e) {
		cerr << src_name << ":" << e.i << "," << e.j << ": " << e.what() << endl;
		exit(1);
	}

	// pgm.save("out");

	state s(&pgm);
	s.read_input(in_name);
	s.debug = opt_debug;
	s.run();
	if (opt_regs)
		s.show_regs();
	s.write_output("/dev/stdout");

	return 0;
}
